!!****f* m_errors/leave_new
!! NAME
!!  leave_new
!!
!! FUNCTION
!!  Routine for clean exit of f90 code, taking into account possible parallelization.
!!
!!  Note the this routine is private and should never be called explictly. 
!!  Please, use the macros:
!!    MSG_ERROR, MSG_BUG 
!!  defined in abi_common.h to abort the execution.
!!  XG : this is not true, in very rare cases, ABINIT has to exit without giving an error (e.g. for non-zero prtkpt )
!!
!! INPUTS
!!  exit_status=(optional, default=1 or -1, see below) the return code of the routine
!!  mode_paral=
!!   'COLL' if all procs are calling the routine with the same message to be
!!     written once only or
!!   'PERS' if the procs are calling the routine with different mesgs
!!     each to be written, or if one proc is calling the routine
!!  print_config=(optional, default=true)
!!       if true print out several informations before leaving
!!
!! OUTPUT
!!  (only writing, then stop)
!!
!! NOTES
!!  By default, it uses "call exit(1)", that is not completely portable.
!!
!! PARENTS
!!      m_errors,m_matlu,m_paw_dmft,qmc_prep_ctqmc,spectral_function,testkgrid
!!
!! CHILDREN
!!      exit
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"


subroutine leave_new(mode_paral,exit_status,print_config)

 use defs_basis
 use m_xmpi
 use m_build_info,      only : dump_config

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'leave_new'
 use interfaces_14_hidewrite
 use interfaces_16_hideleave, except_this_one => leave_new
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 character(len=4),intent(in) :: mode_paral
 integer,intent(in),optional :: exit_status
 logical,intent(in),optional :: print_config

!Local variables-------------------------------
 integer :: nproc
 logical :: print_config_
 !character(len=500) :: msg

! **********************************************************************
! Caveat: Do not use MPI collective calls!
 call wrtout(std_out,ch10//' leave_new : decision taken to exit ...','PERS')

 if (mode_paral == "COLL") then
   call wrtout(std_out,"Why are you using COLL? Are you sure that ALL the processors are calling leave_new?")
 end if

 print_config_=.true.; if (present(print_config)) print_config_=print_config

!Determine nproc in COMM_WORLD
 nproc = xcomm_size(xmpi_world)

!Dump configuration before exiting
 if (print_config_) then
   call print_kinds()
   call xmpi_show_info()
   call dump_config()
 end if

 if (nproc==1) then

   if (present(exit_status)) then
     call leave_myproc(exit_status=exit_status,option=1)
   else
     call leave_myproc(option=1)
   end if

 else
   call wrtout(std_out,'calling XMPI_ABORT...','PERS')
   call leave_myproc(option=1)
 end if

 call xmpi_abort()

end subroutine leave_new
!!***

!----------------------------------------------------------------------

!!****f* m_errors/leave_myproc
!! NAME
!! leave_myproc
!!
!! FUNCTION
!! Routine for clean exit of f90 code by one processor
!!
!! INPUTS
!!   option=(optional argument, default=0)
!!          0: closes output file, then stops
!!          1: only closes output file
!!          2: only stops
!!
!! OUTPUT
!!  (only writing, then stop)
!!
!! NOTES
!!  By default, it uses "call exit(1)", that is not completely portable.
!!
!! PARENTS
!!      leave_new
!!
!! CHILDREN
!!      exit
!!
!! SOURCE

subroutine leave_myproc(exit_status,option)

 use defs_basis
#ifdef FC_NAG
 use f90_unix
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'leave_myproc'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer, intent(in),optional :: exit_status,option

!Local variables-------------------------------
!scalars
 integer :: option_
 logical :: testopen

! **********************************************************************

 option_=0;if(present(option)) option_=option

 if (option_==0.or.option_==1) then
   inquire(ab_out,OPENED=testopen)
   if (testopen) close(ab_out)
 end if

 if (option_==0.or.option_==2) then
#if defined FC_NAG
   if(present(exit_status))then
     call exit(exit_status)
   else
     call exit(-1)
   end if
#elif defined HAVE_FC_EXIT
   if(present(exit_status))then
     call exit(exit_status)
   else
     call exit(1)
   end if
#else
   if(present(exit_status))then
!    stop exit_status !MT 06-2013: Not correct because stop function only accept parameters !
     if (exit_status== 0) stop  "0"
     if (exit_status== 1) stop  "1"
     if (exit_status==-1) stop "-1"
   else
     stop 1
   end if
#endif
 end if

end subroutine leave_myproc
!!***
