!{\src2tex{textfont=tt}}
!!****m* ABINIT/m_distribfft
!! NAME
!!  m_distribfft
!!
!! FUNCTION
!!  This module provides the definition of the different arrays 
!!  used for fft parallelization and n2 plane sharing
!!
!! COPYRIGHT
!! Copyright (C) 2011-2014 ABINIT group (FD,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

MODULE m_distribfft

 use m_profiling
 use defs_basis
 use m_errors

 implicit none

 private

!public procedures.
 public :: init_distribfft
 public :: init_distribfft_seq
 public :: destroy_distribfft
 public :: nullify_distribfft
 public :: copy_distribfft
!!***

!!****t* m_distribfft/distribfft_type
!! NAME
!! distribfft_type
!!
!! FUNCTION
!! The distribfft_type structured datatype gather different information
!! for plane sharing for FFT parallelization
!!
!! SOURCE

 type, public :: distribfft_type

! WARNING : if you modify this datatype, please check whether there might be creation/destruction/copy routines,
! declared in another part of ABINIT, that might need to take into account your modification.

  integer ::  n2_coarse
  integer ::  n2_fine

  integer, pointer :: tab_fftwf2_distrib(:)     !indicates fft processors which owns fft planes in 2nd dimension for fourwf

  integer, pointer :: tab_fftdp2_distrib(:)     !indicates fft processors which owns fft planes in 2nd dimension for fourdp
  integer, pointer :: tab_fftdp3_distrib(:)     !indicates fft processors which owns fft planes in 3rd dimension for fourdp
  integer, pointer :: tab_fftwf2dg_distrib(:) !indicates fft processors which owns fft planes in 2nd dimension for fourwf on fine grid
  integer, pointer :: tab_fftdp2dg_distrib(:) !indicates fft processors which owns fft planes in 2nd dimension for fourdp on fine grid
  integer, pointer :: tab_fftdp3dg_distrib(:) !indicates fft processors which owns fft planes in 3rd dimension for fourdp on fine grid

  integer, pointer :: tab_fftwf2_local(:)       !indicates local i2 indices in fourwf

  integer, pointer :: tab_fftdp2_local(:)       !indicates local i2 indices in fourdp
  integer, pointer :: tab_fftdp3_local(:)       !indicates local i3 indices in fourdp
  integer, pointer :: tab_fftwf2dg_local(:)   !indicates local i2 indices in fourwf on fine grid
  integer, pointer :: tab_fftdp2dg_local(:)   !indicates local i2 indices in fourdp on fine grid
  integer, pointer :: tab_fftdp3dg_local(:)   !indicates local i3 indices in fourdp on fine grid

end type distribfft_type
!!***

CONTAINS

!===========================================================
!!***

!!****f* m_distribfft/init_distribfft
!! NAME
!!  init_distribfft
!!
!! FUNCTION
!!  Initializes the mpi information for FFT distribution
!!
!! COPYRIGHT
!!  Copyright (C) 2002-2014 ABINIT group (FD)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  For the initials of contributors, see
!!  ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!! grid      = 'c' or 'f' for informations about coarse or fine fft grid
!! nproc_fft = number of process used to distribute the fft
!! n2,n3     = sizes of second and third fft grid
!! wf_mode   = (optional) 'b' or 'c' whereas we used a bloc or cyclic distribution mode 
!!
!! SIDE EFFECTS
!!  distribfft = instance of t_distribfft to initialize
!!  Update of "fft distrib" tabs accordingly to the fft parallelisation
!!
!! TODO
!!
!! PARENTS
!!      m_fft_prof,m_qparticles,m_wfs,mpi_setup,scfcv,suscep,vtorhorec
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine init_distribfft(distribfft_arg,grid,nproc_fft,n2,n3,wf_mode)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'init_distribfft'
!End of the abilint section

 implicit none


!Arguments ------------------------------------
!scalars
 integer              , intent(in)          :: nproc_fft,n2,n3
 character(len=1)     , intent(in)          :: grid
 character(len=1)     , intent(in),optional :: wf_mode
 type(distribfft_type), intent(inout)       :: distribfft_arg

!Local variables-------------------------------
!scalars
 integer :: i2,i3, n2_local,n3_local
 character(len=1) :: my_wf_mode
 character(len=500) :: msg

! ***********************************************************************

 DBG_ENTER("COLL")

 my_wf_mode = 'c' !Default mode is cyclic (Goedecker routines)
 if(present(wf_mode)) then
    my_wf_mode = wf_mode
 end if

 !local sizes
 n2_local = n2 / nproc_fft
 n3_local = n3 / nproc_fft

 select case (grid)
 case ('c')
    ! updating informations about coarse fft grid
    if(distribfft_arg%n2_coarse > 0) then
       if(n2 == distribfft_arg%n2_coarse) then
          MSG_WARNING("The distribfft passed was already allocated for coarse grid on the same size")
          return
       else
          MSG_ERROR("The distribfft passed was already allocated for coarse grid")
       endif
    end if
    distribfft_arg%n2_coarse = n2
    !Initialisation of fft distrib tab
    ABI_ALLOCATE(distribfft_arg%tab_fftwf2_distrib,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftwf2_local,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp2_distrib,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp2_local,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp3_distrib,(n3))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp3_local,(n3))
    do i2=1, n2
       select case (my_wf_mode)
       case ('c')
          !cyclic distribution of ig2 planes over fft processors (used by sg fft routines for example)
          distribfft_arg%tab_fftwf2_distrib(i2) = modulo((i2-1),nproc_fft)
          distribfft_arg%tab_fftwf2_local(i2)    = (i2-1)/nproc_fft + 1
       case ('b')
          !bloc distribution of ig2 planes over fft processors  (used by fftwmpi for example)
          distribfft_arg%tab_fftwf2_distrib(i2) = (i2-1) /  n2_local
          distribfft_arg%tab_fftwf2_local(i2)   = modulo((i2-1),n2_local) + 1
       case default
          msg = "Unknown wf_mode for repartition! Only 'c' for cyclic and 'b' for bloc distribution are allowed" 
          MSG_ERROR(msg)
       end select
       !bloc distribution of i2 planes over fft processors for fourdp
       distribfft_arg%tab_fftdp2_distrib(i2) = (i2-1) /  n2_local
       distribfft_arg%tab_fftdp2_local(i2)   = modulo((i2-1),n2_local) + 1
    end do
    do i3=1, n3
       !bloc distribution of i3 planes over fft processors for fourdp
       distribfft_arg%tab_fftdp3_distrib(i3) = (i3-1) /  n3_local
       distribfft_arg%tab_fftdp3_local(i3)   = modulo((i3-1),n3_local) + 1
    end do

 case ('f')
    if(distribfft_arg%n2_fine > 0) then
       if(n2 == distribfft_arg%n2_fine) then
          MSG_WARNING("The distribfft passed was already allocated for fine grid on the same size")
          return
       else
          MSG_ERROR("The distribfft passed was already allocated for fine grid")
       end if
    endif
    distribfft_arg%n2_fine = n2
    ! updating informations about fine fft grid
    ABI_ALLOCATE(distribfft_arg%tab_fftwf2dg_distrib,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftwf2dg_local,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp2dg_distrib,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp2dg_local,(n2))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp3dg_distrib,(n3))
    ABI_ALLOCATE(distribfft_arg%tab_fftdp3dg_local,(n3))
    do i2=1, n2
       select case (my_wf_mode)
       case ('c')
          !cyclic distribution of ig2 planes over fft processors (used by sg fft routines for example)
          distribfft_arg%tab_fftwf2dg_distrib(i2) = modulo((i2-1),nproc_fft)
          distribfft_arg%tab_fftwf2dg_local(i2)    = (i2-1)/nproc_fft + 1
       case ('b')
          !bloc distribution of ig2 planes over fft processors  (used by fftwmpi for example)
          distribfft_arg%tab_fftwf2dg_distrib(i2) = (i2-1) /  n2_local
          distribfft_arg%tab_fftwf2dg_local(i2)   = modulo((i2-1),n2_local) + 1
       case default
          msg = "Unknown wf_mode for repartition! Only 'c' for cyclic and 'b' for bloc distribution are allowed"
          MSG_ERROR(msg)
       end select
       !bloc distribution of i2 planes over fft processors for fourdp on fine grid
       distribfft_arg%tab_fftdp2dg_distrib(i2) = (i2-1) /  n2_local
       distribfft_arg%tab_fftdp2dg_local(i2)   = modulo((i2-1),n2_local) + 1
    end do
    do i3=1, n3
       !bloc distribution of i3 planes over fft processors for fourdp on fine grid
       distribfft_arg%tab_fftdp3dg_distrib(i3) = (i3-1) /  n3_local
       distribfft_arg%tab_fftdp3dg_local(i3)   = modulo((i3-1),n3_local) + 1
    end do
 case default
    MSG_ERROR("Unknown kind of fft grid! Only 'c' for coarse grid and 'f' for fine grid are allowed")
 end select

 DBG_EXIT("COLL")

end subroutine init_distribfft
!!***

!===========================================================

!!****f* m_distribfft/init_distribfft_seq
!! NAME
!!  init_distribfft_seq
!!
!! FUNCTION
!!  Initializes a sequential FFT distribution
!!
!! COPYRIGHT
!!  Copyright (C) 2002-2014 ABINIT group (FD,MT)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  For the initials of contributors, see
!!  ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!! grid  = 'c' or 'f' for informations about coarse or fine fft grid
!! n2,n3 = sizes of second and third fft grid
!! type_four = 'fourdp' or 'fourwf' or 'all' to prepare a call to fourdp/fourwf
!!
!! SIDE EFFECTS
!!  distribfft = instance of t_distribfft to initialize
!!  Update of "fft distrib" tabs accordingly to the fft parallelisation
!!
!! TODO
!!
!! PARENTS
!!      atm2fft,atm2fft3,bethe_salpeter,calc_vhxc_me,dieltcel,ks_ddiago
!!      m_fft_prof,m_gsphere,m_ppmodel,m_screening,m_wfk,make_efg_el,overlap_wf
!!      pawgrnl,pawmknhat,pawmknhat_psipsi,pawsushat,scfcv,screening,sigma
!!      suscep_stat,susk,wffile,wfread,xc_kernel
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine init_distribfft_seq(distribfft_arg,grid,n2,n3,type_four)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'init_distribfft_seq'
!End of the abilint section

 implicit none


!Arguments ------------------------------------
!scalars
 integer              , intent(in)    :: n2,n3
 character(len=1)     , intent(in)    :: grid
 character(len=*)     , intent(in)    :: type_four
 type(distribfft_type), intent(inout) :: distribfft_arg

!Local variables-------------------------------
!scalars
 integer :: ii

! ***********************************************************************

 DBG_ENTER("COLL")

 select case (grid)
 case ('c')
   distribfft_arg%n2_coarse = n2
   if (type_four=='fourwf'.or.type_four(1:3)=='all') then
     if (associated(distribfft_arg%tab_fftwf2_distrib)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftwf2_distrib)
     end if
     if (associated(distribfft_arg%tab_fftwf2_local)) then
     ABI_DEALLOCATE(distribfft_arg%tab_fftwf2_local)
     end if
     ABI_ALLOCATE(distribfft_arg%tab_fftwf2_distrib,(n2))
     ABI_ALLOCATE(distribfft_arg%tab_fftwf2_local,(n2))
     distribfft_arg%tab_fftwf2_distrib=0
     distribfft_arg%tab_fftwf2_local=(/(ii,ii=1,n2)/)
   end if
   if (type_four=='fourdp'.or.type_four(1:3)=='all') then
     if (associated(distribfft_arg%tab_fftdp2_distrib)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp2_distrib)
     end if
     if (associated(distribfft_arg%tab_fftdp2_local)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp2_local)
     end if
     if (associated(distribfft_arg%tab_fftdp3_distrib)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp3_distrib)
     end if
     if (associated(distribfft_arg%tab_fftdp3_local)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp3_local)
     end if
     ABI_ALLOCATE(distribfft_arg%tab_fftdp2_distrib,(n2))
     ABI_ALLOCATE(distribfft_arg%tab_fftdp2_local,(n2))
     ABI_ALLOCATE(distribfft_arg%tab_fftdp3_distrib,(n3))
     ABI_ALLOCATE(distribfft_arg%tab_fftdp3_local,(n3))
     distribfft_arg%tab_fftdp2_distrib=0
     distribfft_arg%tab_fftdp3_distrib=0
     distribfft_arg%tab_fftdp2_local=(/(ii,ii=1,n2)/)
     distribfft_arg%tab_fftdp3_local=(/(ii,ii=1,n3)/)
   end if

 case ('f')
   distribfft_arg%n2_fine = n2
   if (type_four=='fourwf'.or.type_four(1:3)=='all') then
     if (associated(distribfft_arg%tab_fftwf2dg_distrib)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftwf2dg_distrib)
     end if
     if (associated(distribfft_arg%tab_fftwf2dg_local)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftwf2dg_local)
     end if
     ABI_ALLOCATE(distribfft_arg%tab_fftwf2dg_distrib,(n2))
     ABI_ALLOCATE(distribfft_arg%tab_fftwf2dg_local,(n2))
     distribfft_arg%tab_fftwf2dg_distrib=0
     distribfft_arg%tab_fftwf2dg_local=(/(ii,ii=1,n2)/)
   end if
   if (type_four=='fourdp'.or.type_four(1:3)=='all') then
     if (associated(distribfft_arg%tab_fftdp2dg_distrib)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp2dg_distrib)
     end if
     if (associated(distribfft_arg%tab_fftdp2dg_local)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp2dg_local)
     end if
     if (associated(distribfft_arg%tab_fftdp3dg_distrib)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp3dg_distrib)
     end if
     if (associated(distribfft_arg%tab_fftdp3dg_local)) then
       ABI_DEALLOCATE(distribfft_arg%tab_fftdp3dg_local)
     end if
     ABI_ALLOCATE(distribfft_arg%tab_fftdp2dg_distrib,(n2))
     ABI_ALLOCATE(distribfft_arg%tab_fftdp2dg_local,(n2))
     ABI_ALLOCATE(distribfft_arg%tab_fftdp3dg_distrib,(n3))
     ABI_ALLOCATE(distribfft_arg%tab_fftdp3dg_local,(n3))
     distribfft_arg%tab_fftdp2dg_distrib=0
     distribfft_arg%tab_fftdp3dg_distrib=0
     distribfft_arg%tab_fftdp2dg_local=(/(ii,ii=1,n2)/)
     distribfft_arg%tab_fftdp3dg_local=(/(ii,ii=1,n3)/)
   end if

 case default
    MSG_ERROR("Unknown kind of fft grid! Only 'c' for coarse grid and 'f' for fine grid are allowed")
 end select

 DBG_EXIT("COLL")

end subroutine init_distribfft_seq
!!***

!===========================================================

!!****f* m_distribfft/destroy_distribfft
!! NAME
!!  destroy_distribfft
!!
!! FUNCTION
!!  Cleans-up the mpi informations for FFT distribution
!!  (mostly deallocate parts distribfft(:) ).
!!
!! COPYRIGHT
!!  Copyright (C) 2002-2014 ABINIT group (FD)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/sCOPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  For the initials of contributors, see
!!  ~abinit/doc/developers/contributors.txt.
!!
!! SIDE EFFECTS
!!  mpi_enreg=informations about MPI parallelization
!!
!! TODO
!!
!! PARENTS
!!      atm2fft,atm2fft3,m_qparticles,m_wfs,mpi_enreg_tools,pawgrnl,pawmknhat
!!      pawmknhat_psipsi,pawsushat,scfcv,vtorhorec,wffile
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine destroy_distribfft(distribfft_arg)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'destroy_distribfft'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(distribfft_type), intent(inout) :: distribfft_arg

!Local variables-------------------------------

! ***********************************************************************

 DBG_ENTER("COLL")

 distribfft_arg%n2_coarse=0
 distribfft_arg%n2_fine  =0

 if (associated(distribfft_arg%tab_fftwf2_distrib)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftwf2_distrib)
    nullify(distribfft_arg%tab_fftwf2_distrib)
 end if

 if (associated(distribfft_arg%tab_fftdp2_distrib)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp2_distrib)
    nullify(distribfft_arg%tab_fftdp2_distrib)
 end if
 if (associated(distribfft_arg%tab_fftdp3_distrib)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp3_distrib)
    nullify(distribfft_arg%tab_fftdp3_distrib)
 end if
 if (associated(distribfft_arg%tab_fftwf2dg_distrib)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftwf2dg_distrib)
    nullify(distribfft_arg%tab_fftwf2dg_distrib)
 end if
 if (associated(distribfft_arg%tab_fftdp2dg_distrib)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp2dg_distrib)
    nullify(distribfft_arg%tab_fftdp2dg_distrib)
 end if
 if (associated(distribfft_arg%tab_fftdp3dg_distrib)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp3dg_distrib)
    nullify(distribfft_arg%tab_fftdp3dg_distrib)
 end if

 if (associated(distribfft_arg%tab_fftwf2_local)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftwf2_local)
    nullify(distribfft_arg%tab_fftwf2_local)
 end if

 if (associated(distribfft_arg%tab_fftdp2_local)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp2_local)
    nullify(distribfft_arg%tab_fftdp2_local)
 end if
 if (associated(distribfft_arg%tab_fftdp3_local)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp3_local)
    nullify(distribfft_arg%tab_fftdp3_local)
 end if
 if (associated(distribfft_arg%tab_fftwf2dg_local)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftwf2dg_local)
    nullify(distribfft_arg%tab_fftwf2dg_local)
 end if
 if (associated(distribfft_arg%tab_fftdp2dg_local)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp2dg_local)
    nullify(distribfft_arg%tab_fftdp2dg_local)
 end if
 if (associated(distribfft_arg%tab_fftdp3dg_local)) then
    ABI_DEALLOCATE(distribfft_arg%tab_fftdp3dg_local)
    nullify(distribfft_arg%tab_fftdp3dg_local)
 end if

 DBG_EXIT("COLL")

end subroutine destroy_distribfft
!!***

!===========================================================

!!****f* m_distribfft/nullify_distribfft
!! NAME
!!  nullify_distribfft
!!
!! FUNCTION
!!  Cleans-up the mpi informations for FFT distribution
!!  (mostly deallocate parts distribfft(:) ).
!!
!! COPYRIGHT
!!  Copyright (C) 2002-2014 ABINIT group (FD)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/sCOPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  For the initials of contributors, see
!!  ~abinit/doc/developers/contributors.txt.
!!
!! SIDE EFFECTS
!!  mpi_enreg=informations about MPI parallelization
!!
!! TODO
!!
!! PARENTS
!!      atm2fft,atm2fft3,initmpi_seq,mpi_enreg_tools,pawgrnl,pawmknhat
!!      pawmknhat_psipsi,pawsushat
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine nullify_distribfft(distribfft_arg)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'nullify_distribfft'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(distribfft_type), intent(inout) :: distribfft_arg

!Local variables-------------------------------

! ***********************************************************************

 DBG_ENTER("COLL")

 distribfft_arg%n2_coarse=0
 distribfft_arg%n2_fine  =0
 nullify(distribfft_arg%tab_fftwf2_distrib)
 nullify(distribfft_arg%tab_fftdp2_distrib)
 nullify(distribfft_arg%tab_fftdp3_distrib)
 nullify(distribfft_arg%tab_fftwf2dg_distrib)
 nullify(distribfft_arg%tab_fftdp2dg_distrib)
 nullify(distribfft_arg%tab_fftdp3dg_distrib)
 nullify(distribfft_arg%tab_fftwf2_local)
 nullify(distribfft_arg%tab_fftdp2_local)
 nullify(distribfft_arg%tab_fftdp3_local)
 nullify(distribfft_arg%tab_fftwf2dg_local)
 nullify(distribfft_arg%tab_fftdp2dg_local)
 nullify(distribfft_arg%tab_fftdp3dg_local)

 DBG_EXIT("COLL")

end subroutine nullify_distribfft
!!***

!===========================================================

!!****f* m_distribfft/copy_distribfft
!! NAME
!!  copy_distribfft
!!
!! FUNCTION
!!  Cleans-up the mpi informations for FFT distribution
!!  (mostly deallocate parts distribfft(:) ).
!!
!! COPYRIGHT
!!  Copyright (C) 2002-2014 ABINIT group (FD)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/sCOPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!  For the initials of contributors, see
!!  ~abinit/doc/developers/contributors.txt.
!!
!! SIDE EFFECTS
!!  mpi_enreg=informations about MPI parallelization
!!
!! TODO
!!
!! PARENTS
!!      mpi_enreg_tools
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine copy_distribfft(distribfft_src, distribfft_dst)


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'copy_distribfft'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(distribfft_type), intent(in)   :: distribfft_src
 type(distribfft_type), intent(out) :: distribfft_dst

!Local variables-------------------------------

! ***********************************************************************

 DBG_ENTER("COLL")

 distribfft_dst%n2_coarse=  distribfft_src%n2_coarse
 distribfft_dst%n2_fine  =  distribfft_src%n2_fine

 if (associated(distribfft_src%tab_fftwf2_distrib)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftwf2_distrib,(size(distribfft_src%tab_fftwf2_distrib)))
     distribfft_dst%tab_fftwf2_distrib=distribfft_src%tab_fftwf2_distrib
 end if
  if (associated(distribfft_src%tab_fftdp2_distrib)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp2_distrib,(size(distribfft_src%tab_fftdp2_distrib)))
     distribfft_dst%tab_fftdp2_distrib=distribfft_src%tab_fftdp2_distrib
  end if
  if (associated(distribfft_src%tab_fftdp3_distrib)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp3_distrib,(size(distribfft_src%tab_fftdp3_distrib)))
     distribfft_dst%tab_fftdp3_distrib=distribfft_src%tab_fftdp3_distrib
  end if
  if (associated(distribfft_src%tab_fftwf2dg_distrib)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftwf2dg_distrib,(size(distribfft_src%tab_fftwf2dg_distrib)))
     distribfft_dst%tab_fftwf2dg_distrib=distribfft_src%tab_fftwf2dg_distrib
  end if
  if (associated(distribfft_src%tab_fftdp2dg_distrib)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp2dg_distrib,(size(distribfft_src%tab_fftdp2dg_distrib)))
     distribfft_dst%tab_fftdp2dg_distrib=distribfft_src%tab_fftdp2dg_distrib
  end if
  if (associated(distribfft_src%tab_fftdp3dg_distrib)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp3dg_distrib,(size(distribfft_src%tab_fftdp3dg_distrib)))
     distribfft_dst%tab_fftdp3dg_distrib=distribfft_src%tab_fftdp3dg_distrib
  end if
  if (associated(distribfft_src%tab_fftwf2_local)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftwf2_local,(size(distribfft_src%tab_fftwf2_local)))
     distribfft_dst%tab_fftwf2_local=distribfft_src%tab_fftwf2_local
  end if

  if (associated(distribfft_src%tab_fftdp2_local)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp2_local,(size(distribfft_src%tab_fftdp2_local)))
     distribfft_dst%tab_fftdp2_local=distribfft_src%tab_fftdp2_local
  end if
  if (associated(distribfft_src%tab_fftdp3_local)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp3_local,(size(distribfft_src%tab_fftdp3_local)))
     distribfft_dst%tab_fftdp3_local=distribfft_src%tab_fftdp3_local
  end if
  if (associated(distribfft_src%tab_fftwf2dg_local)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftwf2dg_local,(size(distribfft_src%tab_fftwf2dg_local)))
     distribfft_dst%tab_fftwf2dg_local=distribfft_src%tab_fftwf2dg_local
  end if
  if (associated(distribfft_src%tab_fftdp2dg_local)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp2dg_local,(size(distribfft_src%tab_fftdp2dg_local)))
     distribfft_dst%tab_fftdp2dg_local=distribfft_src%tab_fftdp2dg_local
  end if
  if (associated(distribfft_src%tab_fftdp3dg_local)) then
     ABI_ALLOCATE(distribfft_dst%tab_fftdp3dg_local,(size(distribfft_src%tab_fftdp3dg_local)))
     distribfft_dst%tab_fftdp3dg_local=distribfft_src%tab_fftdp3dg_local
  end if

 DBG_EXIT("COLL")

end subroutine copy_distribfft
!!***

END MODULE m_distribfft
!!***
