!{\src2tex{textfont=tt}}
!!****f* ABINIT/fftwmpi_fourdp
!! NAME
!!  fftwmpi_fourdp
!!
!! FUNCTION
!! Driver routine for 3D FFT of lengths nx, ny, nz. Mainly used for densities or potentials.
!! FFT Transform is out-of-place
!!
!! COPYRIGHT
!! Copyright (C) 1998-2014 ABINIT group (FD)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!! cplex=1 if fofr is real, 2 if fofr is complex
!! nx,ny,nz=Number of point along the three directions.
!! isign= +1 : fofg(G) => fofr(R); 
!!        -1 : fofr(R) => fofg(G)
!! fofg(2,nx*ny*nz)=The array to be transformed.
!! [fftw_flags]=Flags used to create the plan. They can be combined with the "+" operator. Defaults to ABI_FFTW_ESTIMATE.
!!
!! OUTPUT 
!! fofr(cplex,nx*ny*nz)=The FFT of fofg
!!
!! NOTES
!!  
!!
!! PARENTS
!!      fourdp
!!
!! CHILDREN
!!      fftwmpi_c2r_tr,fftwmpi_many_dft_ip,fftwmpi_r2c_tr
!!
!! SOURCE
  
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

 subroutine fftwmpi_fourdp(cplex,mpi_enreg,nx,ny,nz,isign,fofg,fofr,fftw_flags)

  use defs_basis
  use defs_abitypes
  use m_profiling
  use m_fftw3
  use m_errors

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_fourdp'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: cplex,nx,ny,nz,isign
 integer,optional,intent(in) :: fftw_flags
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 real(dp),intent(inout) :: fofg(2,nx*ny*nz)
 real(dp),intent(inout) :: fofr(cplex*nx*ny*nz)

!Local variables-------------------------------
!scalars
 integer :: fidx,ii,jj,kk
 integer :: ldx,ldy,ldz,my_flags
!arrays
 real(dp),allocatable :: work1(:,:,:,:)

! *************************************************************************

#if defined HAVE_FFT_FFTW3_MPI
 my_flags=ABI_FFTW_ESTIMATE; if (PRESENT(fftw_flags)) my_flags= fftw_flags

 ldx=nx; ldy=ny;ldz=nz;

 select case (cplex)
   case (2) ! Complex to Complex.
!    stop "fftwmpi_dft Not coded yet"
     ABI_ALLOCATE(work1,(2,ldx,ldy,ldz))
     select case (isign)
       case (ABI_FFTW_BACKWARD)  ! +1
!        allocate(work2(2,ldx*ldy*ldz))
         fidx=0
         do kk=1,nz
           do jj=1,ny
             fidx = (jj-1)*nx + (kk-1)*nx*ny
             do ii=1,nx
               work1(:,ii,jj,kk) = fofg(:,ii+fidx)

             end do
           end do
         end do

         call fftwmpi_many_dft_ip(mpi_enreg,nx,ny,nz,ldx,ldy,ldz,1,isign,work1,work1)

         fidx=0
         do kk=1,nz
           do jj=1,ny
             do ii=1,nx
               fidx = fidx+1
               fofr(2*fidx-1) = work1(1,ii,jj,kk)
               fofr(2*fidx  ) = work1(2,ii,jj,kk)
             end do
           end do
         end do

       case (ABI_FFTW_FORWARD) ! -1
         fidx=0
         do kk=1,nz
           do jj=1,ny
             do ii=1,nx
               fidx = fidx+1
               work1(1,ii,jj,kk) = fofr(2*fidx-1)  
               work1(2,ii,jj,kk) = fofr(2*fidx  )  
             end do
           end do
         end do
         call fftwmpi_many_dft_ip(mpi_enreg,nx,ny,nz,ldx,ldy,ldz,1,isign,work1,work1)
         fidx=0
         do kk=1,nz
           do jj=1,ny
             fidx = (jj-1)*nx + (kk-1)*nx*ny
             do ii=1,nx
               fofg(:,ii+fidx) = work1(:,ii,jj,kk) 
             end do
           end do
         end do

       case default
         MSG_BUG("Wrong isign")

     end select
     ABI_DEALLOCATE(work1)

   case (1) ! Real case.
     select case (isign)
       case (ABI_FFTW_FORWARD) ! +1; R --> G 
         call fftwmpi_r2c_tr(mpi_enreg,nx,ny,nz,nx,ny,nz,1,fofr,fofg,fftw_flags=my_flags)
!        call fftwmpi_r2c_op(mpi_enreg,nx,ny,nz,nx,ny,nz,1,fofr,fofg,fftw_flags=my_flags)

       case (ABI_FFTW_BACKWARD) ! -1; G --> R
         call fftwmpi_c2r_tr(mpi_enreg,nx,ny,nz,nx,ny,nz,1,fofg,fofr,fftw_flags=my_flags)
!        call fftwmpi_c2r_op(mpi_enreg,nx,ny,nz,nx,ny,nz,1,fofg,fofr,fftw_flags=my_flags)

       case default
         MSG_BUG("Wrong isign")
     end select

   case default 
     MSG_BUG(" Wrong value for cplex")
 end select

#else

 if (present(fftw_flags)) then
   ABI_UNUSED( fftw_flags )
 end if
 ABI_UNUSED( fofg(1,1) )
 ABI_UNUSED( fofr(1))
 ABI_UNUSED( isign )
 ABI_UNUSED( mpi_enreg%nproc_fft )
 ABI_UNUSED((/ my_flags,fidx,ii,jj,kk /))
 ABI_UNUSED((/ ldx,ldy,ldz/))
 MSG_ERROR("FFTW3_MPI support not activated")
 ABI_UNUSED(work1(1,1,1,1))
#endif

 end subroutine fftwmpi_fourdp
!!***
