!{\src2tex{textfont=tt}}
!!****f* ABINIT/fftwmpi_fourwf
!! NAME
!! fftwmpi_fourwf
!!
!! FUNCTION
!! Carry out composite Fourier transforms between real and reciprocal (G) space.
!! Wavefunctions, contained in a sphere in reciprocal space,
!! can be FFT to real space. They can also be FFT from real space
!! to a sphere. Also, the density maybe accumulated, and a local potential can be applied.
!!
!! The different options are :
!! - option=0 --> reciprocal to real space and output the result.
!! - option=1 --> reciprocal to real space and accumulate the density.
!! - option=2 --> reciprocal to real space, apply the local potential to the wavefunction
!!                in real space and produce the result in reciprocal space.
!! - option=3 --> real space to reciprocal space.
!!                NOTE that in this case, fftalg=1x1 MUST be used. This may be changed in the future.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2014 ABINIT group (FD)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!! cplex= if 1 , denpot is real, if 2 , denpot is complex
!!    (cplex=2 only allowed for option=2, and istwf_k=1)
!!    not relevant if option=0 or option=3, so cplex=0 can be used to minimize memory
!! fofgin(2,npwin)=holds input wavefunction in G vector basis sphere.
!!                 (intent(in) but the routine sphere can modify it for another iflag)
!! gboundin(2*mgfft+8,2)=sphere boundary info for reciprocal to real space
!! gboundout(2*mgfft+8,2)=sphere boundary info for real to reciprocal space
!! istwf_k=option parameter that describes the storage of wfs
!! kg_kin(3,npwin)=reduced planewave coordinates, input
!! kg_kout(3,npwout)=reduced planewave coordinates, output
!! mgfft=maximum size of 1D FFTs
!! mpi_enreg=information about MPI parallelization
!! ndat=number of FFT to do in //
!! ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!! npwin=number of elements in fofgin array (for option 0, 1 and 2)
!! npwout=number of elements in fofgout array (for option 2 and 3)
!! n4,n5,n6=ngfft(4),ngfft(5),ngfft(6), dimensions of fofr.
!! option= if 0: do direct FFT
!!         if 1: do direct FFT, then sum the density
!!         if 2: do direct FFT, multiply by the potential, then do reverse FFT
!!         if 3: do reverse FFT only
!! paral_kgb=Flag related to the kpoint-band-fft parallelism
!! weight_r=weight to be used for the accumulation of the density in real space
!!         (needed only when option=1)
!!
!! weight_i=weight to be used for the accumulation of the density in real space
!!         (needed only when option=1 and (fftalg=4 and fftalgc/=0))
!! fofginb(2,npwin)=holds second input wavefunction in G vector basis sphere.
!!                 (intent(in) but the routine sphere can modify it for another iflag) 
!!                 (for non diagonal occupation)
!! use_ndo = use non diagonal occupations.

!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!! Input/Output
!! for option==0, fofgin(2,npwin*ndat)=holds input wavefunction in G sphere;
!!                fofr(2,n4,n5,n6) contains the output Fourier Transform of fofgin;
!!                no use of denpot, fofgout and npwout.
!! for option==1, fofgin(2,npwin*ndat)=holds input wavefunction in G sphere;
!!                denpot(cplex*n4,n5,n6) contains the input density at input,
!!                and the updated density at output (accumulated);
!!                no use of fofgout and npwout.
!! for option==2, fofgin(2,npwin*ndat)=holds input wavefunction in G sphere;
!!                denpot(cplex*n4,n5,n6) contains the input local potential;
!!                fofgout(2,npwout*ndat) contains the output function;
!! for option==3, fofr(2,n4,n5,n6*ndat) contains the input real space wavefunction;
!!                fofgout(2,npwout*ndat) contains its output Fourier transform;
!!                no use of fofgin and npwin.
!!
!! PARENTS
!!      fourwf
!!
!! CHILDREN
!!      c_f_pointer,fftwmpi_free_work_array,fftwmpi_get_work_array
!!      fftwmpi_many_dft_ip,fftwmpi_many_dft_tr,sphere
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine fftwmpi_fourwf(cplex,denpot,fofgin,fofgout,fofr,gboundin,gboundout,istwf_k,&
     &  kg_kin,kg_kout,mgfft,mpi_enreg,ndat,ngfft,npwin,npwout,n4,n5,n6,option,paral_kgb,weight_r,weight_i,&
     &  use_ndo,fofginb) ! optional Arguments.

  use defs_basis
  use defs_abitypes
  use m_profiling
  use m_cgtools
  use m_fftw3
  use m_errors
  use m_timer
  use m_distribfft
#ifdef HAVE_FC_ISO_C_BINDING
  use iso_c_binding
#else
  use m_iso_c_binding
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fftwmpi_fourwf'
 use interfaces_52_fft_mpi_noabirule
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: cplex,istwf_k,n4,n5,n6,ndat,npwin,npwout,option,paral_kgb,mgfft
 integer,intent(in),optional :: use_ndo
 real(dp),intent(in) :: weight_r,weight_i
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: gboundin(2*mgfft+8,2),gboundout(2*mgfft+8,2)
 integer,intent(in) :: kg_kin(3,npwin),kg_kout(3,npwout),ngfft(18)
 real(dp),intent(inout) :: denpot(cplex*n4,n5,n6),fofgin(2,npwin*ndat)
 real(dp),intent(inout),optional :: fofginb(:,:)
 real(dp),intent(inout) :: fofr(2,n4,n5,n6*ndat)
 real(dp),intent(out) :: fofgout(2,npwout*ndat)

!Local variables-------------------------------
!scalars
 integer :: fftalg,fftalga,fftalgc
 integer :: me_fft,n1,n2,n3,fftcache,nfftot,nproc_fft,nd3proc,nd2proc
 integer :: i1,i2,i3,i2_local,i3_local,idat,ipw
 integer(C_INTPTR_T) :: local_n0, local_n0_offset, local_n0_tr, local_n0_tr_offset
 character(len=500) :: msg
 logical :: luse_ndo,grid_found
 type(C_PTR) :: cdata_f,cdata_r
!arrays
 integer :: shiftg(3),symm(3,3)
 integer(C_INTPTR_T) :: fft_sizes(4)
 integer, pointer :: fftn2_distrib(:)  SET2NULL
 integer, pointer :: fftn3_distrib(:)  SET2NULL
 integer, pointer :: ffti2_local(:)     SET2NULL
 integer, pointer :: ffti3_local(:)     SET2NULL
 complex(C_DOUBLE_COMPLEX), pointer :: data_f(:,:,:), data_r(:,:,:)

! *************************************************************************

 ABI_UNUSED(gboundin(1,1))
 ABI_UNUSED(gboundout(1,1))
 ABI_UNUSED(paral_kgb)

#ifdef HAVE_FFT_FFTW3_MPI 
 if ((ndat/=1).and.(option/=2))  then
   MSG_ERROR("ndat/=1 not coded yet for option /= 2 ")
 end if

 if (istwf_k /=1)  then
   MSG_ERROR("istwf_k/=1 is not allowed for FFTW MPI" )
 end if

 n1=ngfft(1); n2=ngfft(2); n3=ngfft(3); nfftot=n1*n2*n3
 fftalg=ngfft(7); fftalga=fftalg/100; fftalgc=mod(fftalg,10)
 me_fft=ngfft(11); nproc_fft=ngfft(10)
 nd2proc= n2 / nproc_fft
 nd3proc= n3 / nproc_fft
 fftcache=ngfft(8)

 luse_ndo=.false.
 if(present(use_ndo).and.present(fofginb)) then
   if(use_ndo==1) luse_ndo=.true.
 end if
 if(luse_ndo) then
   if((size(fofginb,2)==0).and.(use_ndo==1)) then
     write(msg, '(a,a,a,i4,i5)' )&
&     ' fofginb has a dimension equal to zero and use_ndo==1',ch10,&
&     ' Action : check dimension of fofginb',size(fofginb,2),use_ndo
     MSG_ERROR(msg)
   end if
 end if

 if (luse_ndo) then
   MSG_ERROR(" Non diagonal occupations are not compatible with FFTWMPI")
 end if

 if ( ALL(option /= (/0,1,2,3/)) ) then
   write(msg, '(a,i1,a)' )&
&   '  The option number',option,' is not allowed. Only option=0, 1, 2 or 3 are allowed presently.'
   MSG_ERROR(msg)
 end if

 if( option==1 .and. cplex/=1 )then
   write(msg,'(a,i1)')&
&   ' With the option number 1, cplex must be 1 but it is cplex=',cplex
   MSG_ERROR(msg)
 end if

 if( option==2 .and. (cplex/=1 .and. cplex/=2) )then
   write(msg,'(a,i1)')&
&   ' With the option number 2, cplex must be 1 or 2, but it is cplex=',cplex
   MSG_ERROR(msg)
 end if

!Get the distrib associated with this fft_grid
!=> for i2 and i3 planes
 grid_found=.false.
 if ( n2== mpi_enreg%distribfft%n2_coarse) then
   if( n3 == size(mpi_enreg%distribfft%tab_fftdp3_distrib))then
     fftn2_distrib => mpi_enreg%distribfft%tab_fftwf2_distrib
     ffti2_local => mpi_enreg%distribfft%tab_fftwf2_local
     fftn3_distrib => mpi_enreg%distribfft%tab_fftdp3_distrib
     ffti3_local => mpi_enreg%distribfft%tab_fftdp3_local
     grid_found=.true.
   end if
 end if
 if(( n2 == mpi_enreg%distribfft%n2_fine).and.(.not.(grid_found))) then
   if( n3 == size(mpi_enreg%distribfft%tab_fftdp3dg_distrib) )then
     fftn2_distrib => mpi_enreg%distribfft%tab_fftwf2dg_distrib
     ffti2_local => mpi_enreg%distribfft%tab_fftwf2dg_local
     fftn3_distrib => mpi_enreg%distribfft%tab_fftdp3dg_distrib
     ffti3_local => mpi_enreg%distribfft%tab_fftdp3dg_local
     grid_found=.true.
   end if
 end if
 if(.not.(grid_found)) then
   MSG_ERROR("Unable to find an allocated distrib for this fft grid")
 end if

 if (ANY( option == (/0,1,2/) ))  then
   fft_sizes(1)=n3
   fft_sizes(2)=n2
   fft_sizes(3)=n1
   fft_sizes(4)=ndat

   call fftwmpi_get_work_array(cdata_f,cdata_r,3,n1,n2,n3,ndat,mpi_enreg,&
   local_n0,local_n0_offset, &
   local_n0_tr,local_n0_tr_offset)

!  Associate C pointers to fortran array
!  Tranposed mode :  (x,z,y) in Fourrier's space
   call c_f_pointer(cdata_f, data_f, (/ ndat*n1 , n3 , nd2proc /) )   !   [ fft_sizes(3),fft_sizes(1), local_n0_tr ]
   call c_f_pointer(cdata_r, data_r, (/ ndat*n1 , n2 , nd3proc /) )   !   [ fft_sizes(3),fft_sizes(2), local_n0 ]

   data_f(:,:,:) = czero
   data_r(:,:,:) = czero


!  Sphere's equivalent : put fofg in fft box with respect to parallelism
!$OMP PARALLEL DO PRIVATE(ipw,i1,i2,i2_local,i3,idat)
   do ipw=1,npwin
     i1=kg_kin(1,ipw); if(i1<0)i1=i1+n1 ;i1=i1+1
     i2=kg_kin(2,ipw); if(i2<0)i2=i2+n2 ;i2=i2+1
     i3=kg_kin(3,ipw); if(i3<0)i3=i3+n3 ;i3=i3+1
#ifdef DEBUG_MODE
     if( fftn2_distrib(i2) /= me_fft) then
       MSG_ERROR("FFTWMPI: plane wave not handle by this proc") !Simple check for debug purpose, should be removed
     end if
#endif
     i2_local = ffti2_local(i2)
     do idat=1,ndat
       data_f( idat + ndat*(i1-1),i3, i2_local) =  &
       CMPLX(fofgin(1,ipw + npwin*(idat-1)),fofgin(2,ipw + npwin*(idat-1)),C_DOUBLE_COMPLEX)
     end do
   end do

!  execute back transform
   call fftwmpi_many_dft_tr(mpi_enreg,n1,n2,n3,ndat, &
   ABI_FFTW_BACKWARD,data_f,data_r,ior(ABI_FFTW_ESTIMATE,ABI_FFTW_MPI_TRANSPOSED_IN))

   select case (option)
     case (1)  ! Accumulate density

       do i3=1,n3
         if (fftn3_distrib(i3) == me_fft) then
           i3_local = ffti3_local(i3)
!$OMP PARALLEL DO PRIVATE(i1,i2) COLLAPSE(2)
           do i2=1,n2
             do i1=1,n1
               denpot(i1,i2,i3) = denpot(i1,i2,i3) + &
               weight_r *    real(data_r(i1,i2,i3_local))**2 + &
               weight_i *aimag(data_r(i1,i2,i3_local))**2
             end do
           end do
         end if
       end do
       call fftwmpi_free_work_array(cdata_f,cdata_r)
       RETURN ! option==1

     case (2) !  Apply local potential
       if(cplex==1)then
         do i3=1,n3
           if (fftn3_distrib(i3) == me_fft) then
             i3_local = ffti3_local(i3)
!$OMP PARALLEL DO PRIVATE(i1,i2,idat) COLLAPSE(2)
             do i2=1,n2
               do i1=1,n1
                 do idat=1, ndat
                   data_r(idat + ndat*(i1-1),i2,i3_local) = data_r(idat + ndat*(i1-1),i2,i3_local) * denpot(i1,i2,i3)
                 end do
               end do
             end do
           end if
         end do
       else if(cplex==2)then
         do i3=1,n3
           if (fftn3_distrib(i3) == me_fft) then
             i3_local = ffti3_local(i3)
!$OMP PARALLEL DO PRIVATE(i1,i2,idat) COLLAPSE(2)
             do i2=1,n2
               do i1=1,n1
                 do idat=1, ndat
                   data_r(idat + ndat*(i1-1),i2,i3_local) = data_r(idat + ndat*(i1-1),i2,i3_local)   * &
                   CMPLX(denpot(2*i1-1,i2,i3),denpot(2*i1,i2,i3),C_DOUBLE_COMPLEX)
                 end do
               end do
             end do
           end if
         end do
       end if ! cplex

!      now execute forward transform
       call fftwmpi_many_dft_tr(mpi_enreg,n1,n2,n3,ndat, &
       ABI_FFTW_FORWARD,data_r,data_f,ior(ABI_FFTW_ESTIMATE,ABI_FFTW_MPI_TRANSPOSED_OUT))

!      reconstruct fofg from fft box with respect to parallelism
!$OMP PARALLEL DO PRIVATE(i1,i2,i2_local,i3,idat,ipw)
       do ipw=1,npwout
         i1=kg_kout(1,ipw); if(i1<0)i1=i1+n1 ; i1=i1+1
         i2=kg_kout(2,ipw); if(i2<0)i2=i2+n2 ; i2=i2+1
         i3=kg_kout(3,ipw); if(i3<0)i3=i3+n3 ; i3=i3+1
         i2_local = ffti2_local(i2)
         do idat=1,ndat
           fofgout(1,ipw + npwout*(idat-1)) =    real(data_f(idat + ndat*(i1-1),i3,i2_local)) / (1.d0*n1*n2*n3)
           fofgout(2,ipw + npwout*(idat-1)) =aimag(data_f(idat + ndat*(i1-1),i3,i2_local)) / (1.d0*n1*n2*n3)
         end do
       end do

     case (0)
!      just put the transform data in fofr
!$OMP PARALLEL DO PRIVATE(i1,i2,i3_local) COLLAPSE(2)
       do i3_local=1,nd3proc
         do i2=1,n2
           do i1=1,n1
             fofr(1,i1,i2,i3_local) =    real(data_r(i1,i2,i3_local))
             fofr(2,i1,i2,i3_local) =   aimag(data_r(i1,i2,i3_local))
           end do
         end do
       end do

     case default
       MSG_ERROR("option isn't recognized")

   end select

   call fftwmpi_free_work_array(cdata_f,cdata_r)

 else if (option==3) then !  The data for option==3 is already in fofr.
   MSG_WARNING(" Option 3 has not been fully tested  yet")
   call fftwmpi_many_dft_ip(mpi_enreg,n1,n2,n3,n4,n5,n6,ndat,ABI_FFTW_FORWARD,fofr,fofr)
   shiftg(:)=0
   symm=0; symm(1,1)=1; symm(2,2)=1; symm(3,3)=1
   call sphere(fofgout,ndat,npwout,fofr,n1,n2,n3,n4,n5,n6,kg_kout,istwf_k,-1,mpi_enreg%me_g0,shiftg,symm,one)
 end if

 RETURN

#else
 MSG_ERROR("FFTWMPI supported is not activated; rebuild with  fft-flavor=fftw3-mpi ")

 ABI_UNUSED(denpot(1,1,1))
 ABI_UNUSED(fofgin(1,1))
 ABI_UNUSED(fofgout(1,1))
 if(present(fofginb)) then
   MSG_WARNING( "UNUSED(fofginb )")
 end if
 ABI_UNUSED(fofr(1,1,1,1) )
 ABI_UNUSED(istwf_k)
 ABI_UNUSED(kg_kin(1,1) )
 ABI_UNUSED(kg_kout(1,1) )
 ABI_UNUSED(mpi_enreg%me_fft)
 ABI_UNUSED(ngfft(1))
 ABI_UNUSED(option)
 if(present(use_ndo) ) then
   ABI_UNUSED(use_ndo )
 end if
 ABI_UNUSED((/weight_r,weight_i/))
 ABI_UNUSED((/ fftalg,fftalga,fftalgc/))
 ABI_UNUSED((/ me_fft,n1,n2,n3,fftcache,nfftot,nproc_fft,nd3proc,nd2proc/))
 ABI_UNUSED((/ shiftg(3),symm(3,3)/))
 ABI_UNUSED((/ ipw,idat,i1,i2,i3,i2_local,i3_local/))
 if(.false.) then
   cdata_f =  C_NULL_PTR
   cdata_r = C_NULL_PTR
   local_n0 = 0
   local_n0_offset= 0
   local_n0_tr= 0
   local_n0_tr_offset = 0
   fft_sizes(1) =0
   msg(1:1) = "a"
   luse_ndo=.false.
   grid_found=.false.
 end if
 nullify(data_f)
 nullify(data_r)
 nullify(fftn2_distrib)
 nullify(fftn3_distrib)
 nullify(ffti2_local)
 nullify(ffti3_local)
#endif

end subroutine fftwmpi_fourwf
!!***
