!{\src2tex{textfont=tt}}
!!****m* ABINIT/m_ncfile
!! NAME
!! m_ncfile
!!
!! FUNCTION
!!  This module provides tools for the IO of NETCDF files.
!!
!! COPYRIGHT
!!  Copyright (C) 2008-2014 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! PARENTS
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

MODULE m_ncfile

 use defs_basis
 use m_profiling
 use m_errors
#ifdef HAVE_TRIO_ETSF_IO
 use etsf_io
#endif
#ifdef HAVE_TRIO_NETCDF
 use netcdf
#endif

 implicit none

 private 
!!***

!!****t* m_ncfile/ncfile_t
!! NAME
!! ncfile_t
!! 
!! FUNCTION
!!  File descriptor used for Netcdf files.
!! 
!! SOURCE

 type,public :: ncfile_t
   integer :: ncid                 ! File handle
   integer :: cmode                ! cmode passed to ncfile_create
   integer :: comm                 ! MPI communicator.
   character(len=fnlen) :: fname   ! File name
 end type ncfile_t
!!***

#ifdef HAVE_TRIO_ETSF_IO
 public :: ncfile_create       ! Create a new netcdf file.
 !public :: ncfile_open        ! Close the netcdf file.
 public :: ncfile_close        ! Define the basic dimensions used in ETSF-IO files. 

 ! Helper functions
 public :: ncid_define_basedims

CONTAINS

!!****f* m_ncfile/ncfile_create
!! NAME
!!  ncfile_create
!!
!! FUNCTION
!!  Create a new netcdf file.
!!
!! INPUTS
!!  path=Filepath.
!!  cmode=Creation mode
!!
!! OUTPUT
!!  ncf<ncfile_t>=netcdf file descriptor.
!!  ncerr=Status error.
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function ncfile_create(ncf,path,cmode,comm) result(ncerr)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'ncfile_create'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(ncfile_t),intent(out) :: ncf
 character(len=*),intent(in) :: path
 integer,intent(in) :: cmode
 integer,optional,intent(in) :: comm
 integer :: ncerr

! *********************************************************************

 ncf%fname = path
 ncf%cmode = cmode

 ncerr = nf90_create(path, cmode, ncf%ncid)

 RETURN
 ABI_UNUSED(comm)

end function ncfile_create
!!***

!----------------------------------------------------------------------

!!****f* m_ncfile/ncfile_close
!! NAME
!!  ncfile_close
!!
!! FUNCTION
!!  Close the netcdf file descriptor.
!!
!! INPUTS
!!  ncf<ncfile_t>=netcdf file descriptor.
!!
!! OUTPUT
!!  ncerr=Status error.
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function ncfile_close(ncf) result(ncerr)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'ncfile_close'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(ncfile_t),intent(inout) :: ncf
 integer :: ncerr

! *********************************************************************

 ncerr = nf90_close(ncf%ncid)

end function ncfile_close
!!***

!----------------------------------------------------------------------

!!****f* m_ncfile/ncid_define_basedims
!! NAME
!!  ncid_define_basedims
!!
!! FUNCTION
!!  Define the basic dimensions used in ETSF-IO files. 
!!
!! INPUTS
!!  ncid=Netcdf identifier.
!!
!! OUTPUT
!!
!! PARENTS
!!      m_header,m_phdos,mkphbs
!!
!! CHILDREN
!!      etsf_io_low_set_define_mode,etsf_io_low_write_dim
!!
!! SOURCE

subroutine ncid_define_basedims(ncid) 

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'ncid_define_basedims'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in) :: ncid

!Local variables-------------------------------
!scalars
 logical :: lstat
 type(ETSF_io_low_error) :: Error_data

! *********************************************************************

 call etsf_io_low_set_define_mode(ncid, lstat, Error_data)
 ETSF_CHECK_ERROR(lstat,Error_data)

!Basic ETSF-IO dimensions that should be always present in the file.
 call etsf_io_low_write_dim(ncid,'complex',2,lstat,Error_data=Error_data)  
 ETSF_CHECK_ERROR(lstat,Error_data)

 call etsf_io_low_write_dim(ncid,"symbol_length",2,lstat,Error_data=Error_data)  
 ETSF_CHECK_ERROR(lstat,Error_data)

 call etsf_io_low_write_dim(ncid,'character_string_length',80,lstat,Error_data=Error_data)  
 ETSF_CHECK_ERROR(lstat,Error_data)

 call etsf_io_low_write_dim(ncid,"number_of_cartesian_directions",3,lstat,Error_data=Error_data)  
 ETSF_CHECK_ERROR(lstat,Error_data)

 call etsf_io_low_write_dim(ncid,"number_of_reduced_dimensions",3,lstat,Error_data=Error_data)  
 ETSF_CHECK_ERROR(lstat,Error_data)

 call etsf_io_low_write_dim(ncid,"number_of_vectors",3,lstat,Error_data=Error_data)  
 ETSF_CHECK_ERROR(lstat,Error_data)

end subroutine ncid_define_basedims
!!***
#endif

END MODULE m_ncfile

