!{\src2tex{textfont=tt}}
!!****f* ABINIT/trans_rta
!! NAME
!!  trans_rta
!!
!! FUNCTION
!!  Calculate transport properties (el. cond., el. therm. cond. and Seebeck) 
!!  within the relaxation time approximation.
!!
!! COPYRIGHT
!!  Copyright (C) 2013-2014 ABINIT group (BXu)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  elph_ds=elphon datastructure with data and dimensions
!!  elph_tr_ds
!!  eigenGS = Ground State eigenvalues
!!  max_occ = maximal occupancy for a band
!!  ucvol = Unit cell volume
!!
!! OUTPUT
!!  cond(ntemper)
!!  sbk(ntemper)
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!
!! CHILDREN
!!      ep_el_weights,xsum_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine trans_rta(elph_ds,elph_tr_ds,gprimd,eigenGS,max_occ,ucvol)
    
 use defs_basis
 use defs_elphon
 use m_kptrank
 use m_io_tools
 use m_errors
 use m_profiling
 use m_xmpi

 use m_crystal,    only : crystal_structure

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'trans_rta'
 use interfaces_77_ddb, except_this_one => trans_rta
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(elph_type),intent(inout) :: elph_ds
 type(elph_tr_type), intent(inout) :: elph_tr_ds

 real(dp),intent(in) :: max_occ
 real(dp),intent(in) :: ucvol
 real(dp),intent(in) :: gprimd(3,3)
 real(dp),intent(in) :: eigenGS(elph_ds%nband,elph_ds%k_phon%nkpt,elph_ds%nsppol)

!Local variables-------------------------------
 character(len=500) :: message
 character(len=fnlen) :: fname

 integer :: ierr,iost,ik_this_proc
 integer :: iFSkpt,jband,isppol,itemp,iene
 integer :: ntemper,nsppol,nkpt,nband,nene
 integer :: icomp,jcomp,itensor
 integer :: unit_cond,unit_therm,unit_sbk

 real(dp) :: Temp,factor,dfermide
 real(dp) :: enemin,enemax,deltaene
 real(dp) :: cond,cth,sbk
 real(dp) :: chu_cond,chu_cth,chu_sbk
 real(dp),allocatable :: cond_e(:,:,:,:)
 real(dp),allocatable :: tmp_wtk(:,:,:,:)

 
! *************************************************************************
 
 write(std_out,*) 'trans_rta : enter '

 nsppol   = elph_ds%nsppol
 nband    = elph_ds%ngkkband
 nkpt     = elph_ds%k_phon%nkpt
 ntemper  = elph_ds%ntemper
 nene = 100

 chu_cond = 4.59988159904764*1.0d6
 chu_cth = 1.078637439971599*1.0d4
 chu_sbk = 1.1604504872087024*1.0d4

 ABI_ALLOCATE(cond_e ,(ntemper,nsppol,nkpt,nband))
 ierr = ABI_ALLOC_STAT
 if (ierr /= 0 ) then
   write (message,'(a)')' out of memory in array cond_e '
   MSG_ERROR(message)
 end if

 ABI_ALLOCATE(tmp_wtk,(elph_ds%nband,nkpt,nsppol,nene))
 ierr = ABI_ALLOC_STAT
 if (ierr /= 0 ) then
   write (message,'(a)')' out of memory in array tmp_wtk '
   MSG_ERROR(message)
 end if

 enemax = maxval(eigenGS(elph_ds%maxFSband,:,:))
 enemin = minval(eigenGS(elph_ds%minFSband,:,:))
 deltaene = (enemax-enemin)/dble(nene-1)

 do isppol = 1, nsppol
   call ep_el_weights(elph_ds%ep_b_min, elph_ds%ep_b_max, eigenGS, elph_ds%elphsmear, &
&   enemin, enemax, nene, gprimd, elph_ds%k_phon%irredtoGS, elph_ds%kptrlatt, max_occ, &
!&   elph_ds%minFSband, elph_ds%nband, elph_ds%nFSband, nsppol, elph_ds%telphint, elph_ds%k_phon, tmp_wtk)
&   elph_ds%minFSband, elph_ds%nband, elph_ds%nFSband, nsppol, 1, elph_ds%k_phon, tmp_wtk)
 end do

!get cond_e
 cond_e = zero
 do itemp=1,ntemper  ! runs over termperature in K
   do isppol = 1, nsppol
     do iene = 1, nene
       do ik_this_proc =1,elph_ds%k_phon%my_nkpt
         iFSkpt = elph_ds%k_phon%my_ikpt(ik_this_proc)
         do jband = 1, nband
           do icomp = 1, 3
             do jcomp = 1, 3
               itensor = (icomp-1)*3+jcomp
               cond_e(itemp,isppol,iene,itensor) = cond_e(itemp,isppol,iene,itensor) + &
&               elph_ds%k_phon%tau_k(itemp,isppol,iFSkpt,jband)* &
&               elph_tr_ds%el_veloc(iFSkpt,elph_ds%minFSband+jband-1,icomp,isppol)* &
&               elph_tr_ds%el_veloc(iFSkpt,elph_ds%minFSband+jband-1,jcomp,isppol)* &
&               tmp_wtk(jband,iFSkpt,isppol,iene)
             end do
           end do
         end do ! jband
       end do ! kpt
     end do ! number of energies
   end do ! nsppol
 end do ! ntemperature

 call xsum_mpi (cond_e, xmpi_world, ierr)

 cond_e = cond_e/nkpt

!get transport coefficients

 unit_cond = get_unit()
 fname = trim(elph_ds%elph_base_name) // '_COND'
 open (unit=unit_cond,file=fname,status='unknown',iostat=iost)
 if (iost /= 0) then
   write (message,'(3a)')' trans_rta : ERROR- opening file ',trim(fname),' as new'
   MSG_ERROR(message)
 end if
!print header to conductivity file
 write (unit_cond,*) '#  Conductivity as a function of temperature.'
 write (unit_cond,*) '#  the formalism is isotropic, so non-cubic crystals may be wrong'
 write (unit_cond,*) '#  '
 write (unit_cond,*) '#  Columns are: '
 write (unit_cond,*) '#  temperature[K]   cond[au]   cond [SI]    '
 write (unit_cond,*) '#  '

 unit_therm = get_unit()
 fname = trim(elph_ds%elph_base_name) // '_CTH'
 open (unit=unit_therm,file=fname,status='unknown',iostat=iost)
 if (iost /= 0) then
   write (message,'(3a)')' trans_rta : ERROR- opening file ',trim(fname),' as new'
   MSG_ERROR(message)
 end if
!print header to thermal conductivity file
 write (unit_therm,'(a)') '# Thermal conductivity as a function of temperature.'
 write (unit_therm,'(a)') '#  the formalism is isotropic, so non-cubic crystals may be wrong'
 write (unit_therm,'(a)') '#  '
 write (unit_therm,'(a)') '#  Columns are: '
 write (unit_therm,'(a)') '#  temperature[K]   thermal cond [au]   thermal cond [SI]'
 write (unit_therm,'(a)') '#  '

 unit_sbk = get_unit()
 fname = trim(elph_ds%elph_base_name) // '_SBK'
 open (unit=unit_sbk,file=fname,status='unknown',iostat=iost)
 if (iost /= 0) then
   write (message,'(3a)')' trans_rta : ERROR- opening file ',trim(fname),' as new'
   MSG_ERROR(message)
 end if
!print header to relaxation time file
 write (unit_sbk,*) '# Seebeck Coefficint as a function of temperature.'
 write (unit_sbk,*) '#  the formalism is isotropic, so non-cubic crystals may be wrong'
 write (unit_sbk,*) '#  '
 write (unit_sbk,*) '#  Columns are: '
 write (unit_sbk,*) '#  temperature[K]   S [au]   S [SI]     '
 write (unit_sbk,*) '#  '

 do isppol=1,nsppol
   do icomp=1, 3
     do jcomp=1, 3
       itensor=(icomp-1)*3+jcomp
       write(unit_cond,*) '# Conductivity for isppol, itrten= ', isppol
       write(unit_therm,*) '# Thermal conductivity for isppol, itrten= ', isppol
       write(unit_therm,*) '# Seebeck coefficient for isppol, itrten= ', isppol
       do itemp=1,ntemper
         Temp=elph_ds%tempermin + elph_ds%temperinc*dble(itemp)
         cond = zero
         cth = zero
         sbk = zero
         do iene = 1, nene
           factor = (enemin+(iene-1)*deltaene - elph_ds%fermie)/Temp
           dfermide = EXP(factor)/(Temp*(EXP(factor)+one)**2.0d0)
           cond = cond + cond_e(itemp,isppol,iene,itensor)*dfermide*deltaene
           cth = cth + cond_e(itemp,isppol,iene,itensor)*(enemin+(iene-1)*deltaene - elph_ds%fermie)**2.0d0*dfermide*deltaene
           sbk = sbk + cond_e(itemp,isppol,iene,itensor)*(enemin+(iene-1)*deltaene - elph_ds%fermie)*dfermide*deltaene
         end do
         sbk = sbk/Temp/cond
         cond = cond/ucvol
         cth = cth/Temp/ucvol
         write(unit_cond,'(3D20.10)')Temp,cond,cond*chu_cond
         write(unit_therm,'(3D20.10)')Temp,cth,cth*chu_cth
         write(unit_sbk,'(3D20.10)')Temp,sbk,sbk*chu_sbk
       end do ! temperature
       write(unit_cond,*)
       write(unit_therm,*)
       write(unit_sbk,*)
     end do ! jcomp
   end do ! icomp
 end do !end isppol

 ABI_DEALLOCATE(tmp_wtk)
 ABI_DEALLOCATE(cond_e)

 close (unit=unit_cond)
 close (unit=unit_therm)
 close (unit=unit_sbk)


end subroutine trans_rta
!!***
